/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package core;

/**
 *
 * @author pchr
 */
public class SpecCalc {
    private final double z_value;
    private final double[] thePeriods;
    private final double[] Sdisp;
    private final double[] PSvelc;
    private final double[] PSaccl;
    private final int size;
    private final double[] fext;
    private final double[] fext_interp;
    private final double timeStep;
    private final double Dt;
    double beta=0.25;
    double gamma=0.5;
    
    public SpecCalc(int n, double T1, double T2, double zeta, double[] fext, double Dt, double tstep){
        this.z_value=zeta;
        size = n;
        thePeriods = new double[n];
        Sdisp = new double[n];
        PSvelc = new double[n];
        PSaccl = new double[n];
        
        this.fext=fext;
        this.timeStep=tstep;
        this.Dt=Dt;
        if(tstep>Dt)tstep=Dt;
        
        for(int i=0;i<n;i++){
            thePeriods[i]=T1+(T2-T1)*((double) i)/((double) n-1);
        }
        
        double TotalTime=Dt*fext.length;
        int m=(int) Math.floor(TotalTime/tstep);
//        if( (Dt%tstep)==0)m=m-1;
        
        fext_interp=new double[m];
        fext_interp[0]=fext[0];
        for(int i=1;i<m;i++){
            double t=i*tstep;
            int q=(int) Math.floor(t/Dt);
            if(q<fext.length-1){
                fext_interp[i]=fext[q]+(fext[q+1]-fext[q])*(t-q*Dt)/Dt;
            }else{
                fext_interp[i]=fext[q-1]+(fext[q]-fext[q-1])*(t-(q-1)*Dt)/Dt;
            }
            
//            System.out.println(t+" "+fext_interp[i]);
        }
    }
    
    public void set_b_g(double beta, double gamma){
        this.beta=beta;
        this.gamma=gamma;
    }
    
    public void calc(){
        double[] w= new double[8];
        w[0]=1./(beta*timeStep*timeStep);   w[1]=gamma/(beta*timeStep); w[2]=1.0/(beta*timeStep);
        w[3]=1.0/(2*beta)-1.;               w[4]=gamma/beta-1.;         w[5]=timeStep*(gamma/beta-2.)/2.0;
        w[6]=timeStep*(1.-gamma);           w[7]=timeStep*gamma;
        
        double feff;
        int ii=0;
        PSaccl[0]=0.;
        if(thePeriods[0]==0.){
            for(int j=0; j<fext_interp.length; j++){
                PSaccl[0]=Math.max(PSaccl[0], Math.abs(fext_interp[j]));
            }
            PSvelc[0]=0.;
            Sdisp[0]=0.;
        }
        
        for(int i=1; i<thePeriods.length; i++){
            double k=4.0*Math.PI*Math.PI/(thePeriods[i]*thePeriods[i]);
            double m=1.0;
            double c=4.0*Math.PI*this.z_value/(thePeriods[i]);
            double keff=k+w[0]*m+w[1]*c;
            double u=0.; double v=0.;
            double a=-fext_interp[0];
            Sdisp[i]=u;
            for(int j=1; j<fext_interp.length; j++){
                feff=-fext_interp[j]+m*(w[0]*u+w[2]*v+w[3]*a)
                     +c*(w[1]*u+w[4]*v+w[5]*a);
                double un=feff/keff;
                Sdisp[i]=Math.max(Sdisp[i], Math.abs(un));
                double an=w[0]*(un-u)-w[2]*v-w[3]*a;
                double vn=v+w[6]*a+w[7]*an;
                u=un; a=an; v=vn;
            }
            PSaccl[i]=Sdisp[i]*(2.*Math.PI/thePeriods[i])*(2.*Math.PI/thePeriods[i]);
            PSvelc[i]=Sdisp[i]*(2.*Math.PI/thePeriods[i]);
        }
    }
    
    public int getSize(){return this.size;}
    
    public double[] get_thePeriods(){return this.thePeriods;}
    
    public double[] get_Sdisp(){return this.Sdisp;}
    
    public double[] get_PSvelc(){return this.PSvelc;}
    
    public double[] get_PSaccl(){return this.PSaccl;}
    
    public double[] get_fext(){return this.fext;}
    
    public double get_dt_of_fext(){return this.Dt;}
}
