/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package core;

import Jama.EigenvalueDecomposition;
import Jama.Matrix;
import java.util.Iterator;
import java.util.Map;

/**
 *
 * @author pchr
 */
public class Storey extends Structure{
    private double Lx;
    private double Ly;
    private double p; // maza ana tetragoniko metro
    private double h;
    private double Elast;
    private boolean stiff=false; // false means flexible plate
    
    private Node MassPoint;
    private Node ElastPoint;
    private Node Pole1;
    private Node Pole2;
    private Node Pole3;
    
    private double omegaI,omegaII,omegaIII;
    
    private double ex,ey;
    private double r;
    
    public Storey(){
        this.type=2;
        numDOFS=3;
    }

    @Override
    public void setK() {
        K = new Matrix(3,3); K.init();
        double kxx,kyy,kxz,kyz,kzz;
        double coef;
        if(this.stiff){coef=12.;}else{coef=3.;}
        kxx=0.;kyy=0.;kxz=0.;kyz=0.;kzz=0.;
        for(Iterator<StoreyElement> it=this.StoreyElements.values().iterator(); it.hasNext();){
            StoreyElement theElement = it.next();
            kxx+=coef*Elast*theElement.getIy()/(h*h*h);
            kyy+=coef*Elast*theElement.getIx()/(h*h*h);
            kxz+=-theElement.getPoint().getY()*coef*Elast*theElement.getIy()/(h*h*h);
            kyz+=theElement.getPoint().getX()*coef*Elast*theElement.getIx()/(h*h*h);
            kzz+=(theElement.getPoint().getY()*coef*Elast*theElement.getIy()*theElement.getPoint().getY()+
                    theElement.getPoint().getX()*coef*Elast*theElement.getIx()*theElement.getPoint().getX())/(h*h*h);
        }
        K.set(0, 0, kxx);  K.set(0, 1, 0.);  K.set(0, 2, kxz);
        K.set(1, 0, 0.);   K.set(1, 1, kyy); K.set(1, 2, kyz);
        K.set(2, 0, kxz);  K.set(2, 1, kyz); K.set(2, 2, kzz);
//        K.print(12, 6);
        this.ex=kyz/kyy; this.ey=-kxz/kxx;
        ElastPoint= new Node(444,ex,ey);
    }

    @Override
    public void setM() {
        M = new Matrix(3,3); M.init();
        double m,Jm;
        m=this.Lx*this.Ly*this.p;
        Jm=m*(Lx*Lx+Ly*Ly)/12.;
        r=Math.sqrt(Jm/m);
        M.set(0, 0, m);
        M.set(1, 1, m);
        M.set(2, 2, Jm);
        //M.print(12, 6);
        MassPoint = new Node(444,0.0,0.0);
    }

    @Override
    public void setC(double zeta) {
        Matrix Cd;
        C = new Matrix(3,3);
        Cd = new Matrix(3,3);
        C.init(); Cd.init();
        Cd.set(0, 0, 2*zeta*2.*Math.PI/(this.getEigPeriod(1))*this.getEigMass(1));
        Cd.set(1, 1, 2*zeta*2.*Math.PI/(this.getEigPeriod(2))*this.getEigMass(2));
        Cd.set(2, 2, 2*zeta*2.*Math.PI/(this.getEigPeriod(3))*this.getEigMass(3));
        C=(((this.eigenMat.transpose()).inverse()).times(Cd)).times(this.eigenMat.inverse());
    }

    @Override
    public double getXLength() {
        return this.Lx;
    }

    @Override
    public double getYLength() {
        return this.Ly;
    }

    @Override
    public Matrix getDefElement(int elemId, double x0, int sit, boolean isoFlag) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public Matrix getEigElement(int elemId, double x0, int sit, boolean isoFlag) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public void setGeometry(double[] data) {
        this.Lx=data[0];
        this.Ly=data[1];
        this.h=data[3];
        this.p=data[2];
        this.Elast=data[4];
    }
    
    public void setStiff(boolean theStiff){
        this.stiff=theStiff;
    }
    
    public double getEx(){
        return this.ex;
    }
    
    public double getEy(){
        return this.ey;
    }
    
    public double getR(){
        return this.r;
    }
    
    public double getOmegaI(){
        return this.omegaI;
    }
    
    public double getOmegaII(){
        return this.omegaII;
    }
    
    public double getOmegaIII(){
        return this.omegaIII;
    }
    
    public Node getPole1(){
        return this.Pole1;
    }
    
    public Node getPole2(){
        return this.Pole2;
    }
    
    public Node getPole3(){
        return this.Pole3;
    }
    
    public Node getElastPoint(){
        return this.ElastPoint;
    }
    
    public Node getMassPoint(){
        return this.MassPoint;
    }
    
    @Override
    public void EigenRun(){
//        K.print(12, 6);
        Matrix eVec = new Matrix(numDOFS,1);
        Matrix delta = new Matrix(numDOFS,1);
        delta.init();
        delta.set(0, 0, 1.);
        delta.set(1, 0, 1.);
        delta.set(2, 0, 1.);
        K=K.inverse();
        M=K.times(M);
        this.EVofInvM_A = new EigenvalueDecomposition(M);
        eig_vals = new double[3];
        eigenMat = new Matrix(3,3);
        this.setK();
        this.setM();
        for(int i=0;i<3;i++){
            eig_vals[i]=EVofInvM_A.getRealEigenvalues()[i];
            //eigenMat.set(0, i, EVofInvM_A.getV().get(0, i)/EVofInvM_A.getV().get(2, i));
            //eigenMat.set(1, i, EVofInvM_A.getV().get(1, i)/EVofInvM_A.getV().get(2, i));
            //eigenMat.set(2, i, EVofInvM_A.getV().get(2, i)/EVofInvM_A.getV().get(2, i));
        }
        // SORT
        int e1,e2,e3;
        double max=eig_vals[0];
        e1=0;e2=0;e3=0;
        for(int i=0;i<3;i++){
            if(eig_vals[i]>max){max=eig_vals[i]; e1=i;}
        }
        
        if(e1==0){
            if(eig_vals[1]>eig_vals[2]){e2=1;e3=2;}
            if(eig_vals[2]>eig_vals[1]){e2=2;e3=1;}
        }
        if(e1==1){
            if(eig_vals[0]>eig_vals[2]){e2=0;e3=2;}
            if(eig_vals[2]>eig_vals[0]){e2=2;e3=0;}
        }
        if(e1==2){
            if(eig_vals[0]>eig_vals[1]){e2=0;e3=1;}
            if(eig_vals[1]>eig_vals[0]){e2=1;e3=0;}
        }
        
        eig_vals[0]=EVofInvM_A.getRealEigenvalues()[e1];
        eig_vals[1]=EVofInvM_A.getRealEigenvalues()[e2];
        eig_vals[2]=EVofInvM_A.getRealEigenvalues()[e3];
        
        eigenMat.set(0, 0, EVofInvM_A.getV().get(0, e1));
        eigenMat.set(1, 0, EVofInvM_A.getV().get(1, e1));
        eigenMat.set(2, 0, EVofInvM_A.getV().get(2, e1));
        
        eigenMat.set(0, 1, EVofInvM_A.getV().get(0, e2));
        eigenMat.set(1, 1, EVofInvM_A.getV().get(1, e2));
        eigenMat.set(2, 1, EVofInvM_A.getV().get(2, e2));
        
        eigenMat.set(0, 2, EVofInvM_A.getV().get(0, e3));
        eigenMat.set(1, 2, EVofInvM_A.getV().get(1, e3));
        eigenMat.set(2, 2, EVofInvM_A.getV().get(2, e3));
        
        Pole1 = new Node(1111,-eigenMat.get(1, 0)/eigenMat.get(2, 0),eigenMat.get(0, 0)/eigenMat.get(2, 0));
        Pole2 = new Node(2222,-eigenMat.get(1, 1)/eigenMat.get(2, 1),eigenMat.get(0, 1)/eigenMat.get(2, 1));
        Pole3 = new Node(3333,-eigenMat.get(1, 2)/eigenMat.get(2, 2),eigenMat.get(0, 2)/eigenMat.get(2, 2));
        
        omegaI=Math.sqrt(K.get(0, 0)/M.get(0, 0));
        omegaII=Math.sqrt(K.get(1, 1)/M.get(1, 1));
        omegaIII=Math.sqrt(K.get(2, 2)/M.get(2, 2));
        
        this.EigenMasses = new double[numDOFS];
        this.ContributionCoefs = new double[numDOFS];
        
        for(int i_eig=0; i_eig<numDOFS; i_eig++){
            eVec = eigenMat.getMatrix(0, numDOFS-1, i_eig, i_eig);
            EigenMasses[i_eig]=eVec.transpose().times(M).times(eVec).get(0, 0);
            ContributionCoefs[i_eig]=eVec.transpose().times(M).times(delta).get(0, 0)/EigenMasses[i_eig];
         }
    }
    
    public Node getEigDisplacement(Node thePoint, int theEig){
        double fx = getEigenVector(theEig)[0];
        double fy = getEigenVector(theEig)[1];
        double fz = getEigenVector(theEig)[2];
        double max=0.;
        if(Math.abs(fx)>max)max=Math.abs(fx);
        if(Math.abs(fy)>max)max=Math.abs(fy);
        if(Math.abs(fz)>max)max=Math.abs(fz);
        fx=scale*fx/max; fy=scale*fy/max; fz=scale*fz/max;
        double x=thePoint.getX();
        double y=thePoint.getY();
        //point Displaced = new point(x+(fx-y*fz),y+(fy+x*fz));
        Node Displaced = new Node(0,fx+Math.cos(fz)*x-Math.sin(fz)*y,fy+Math.cos(fz)*y+Math.sin(fz)*x);
        //point Displaced = new point(fx+Math.cos(fz)*x+Math.sin(fz)*y,fy+Math.cos(fz)*y-Math.sin(fz)*x);
        return Displaced;
    }
    
    public Node getDefDisplacement(Node thePoint, int step){
        double fx = this.getDisp1(step);
        double fy = this.getDisp2(step);
        double fz = this.getDisp3(step);
        double max=0.;
        if(Math.abs(fx)>max)max=Math.abs(fx);
        if(Math.abs(fy)>max)max=Math.abs(fy);
        if(Math.abs(fz)>max)max=Math.abs(fz);
        fx=scale*fx/max; fy=scale*fy/max; fz=scale*fz/max;
        double x=thePoint.getX();
        double y=thePoint.getY();
        //point Displaced = new point(x+(fx-y*fz),y+(fy+x*fz));
        Node Displaced = new Node(0,fx+Math.cos(fz)*x-Math.sin(fz)*y,fy+Math.cos(fz)*y+Math.sin(fz)*x);
        //point Displaced = new point(fx+Math.cos(fz)*x+Math.sin(fz)*y,fy+Math.cos(fz)*y-Math.sin(fz)*x);
        return Displaced;
    }
    
    public boolean isElastic(){
        boolean ans=false;
        if(this.ElastPoint!=null)ans=true;
        return ans;
    }
    
    public boolean isMass(){
        boolean ans=false;
        if(this.ElastPoint!=null)ans=true;
        return ans;
    }
    
}
